(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/http'), require('rxjs/Rx'), require('rxjs/operator/map'), require('rxjs/operator/filter'), require('rxjs/operator/catch')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/http', 'rxjs/Rx', 'rxjs/operator/map', 'rxjs/operator/filter', 'rxjs/operator/catch'], factory) :
	(factory((global.ng = global.ng || {}, global.ng['angular4-keycloak'] = global.ng['angular4-keycloak'] || {}),global.ng.core,global.ng.http,global.rxjs_Rx));
}(this, (function (exports,_angular_core,_angular_http,rxjs_Rx) { 'use strict';

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * Default adapter for web browsers
 */
var DefaultAdapter = (function () {
    function DefaultAdapter() {
    }
    DefaultAdapter.prototype.login = function (options) {
        window.location.href = Keycloak.createLoginUrl(options);
    };
    DefaultAdapter.prototype.logout = function (options) {
        window.location.href = Keycloak.createLogoutUrl(options);
    };
    DefaultAdapter.prototype.register = function (options) {
        window.location.href = Keycloak.createRegisterUrl(options);
    };
    DefaultAdapter.prototype.accountManagement = function () {
        window.location.href = Keycloak.createAccountUrl({});
    };
    DefaultAdapter.prototype.redirectUri = function (options, encodeHash) {
        if (arguments.length === 1) {
            encodeHash = true;
        }
        if (options && options.redirectUri) {
            return options.redirectUri;
        }
        else {
            var redirectUri = location.href;
            if (location.hash && encodeHash) {
                redirectUri = redirectUri.substring(0, location.href.indexOf('#'));
                redirectUri += (redirectUri.indexOf('?') === -1 ? '?' : '&') + 'redirect_fragment=' +
                    encodeURIComponent(location.hash.substring(1));
            }
            return redirectUri;
        }
    };
    return DefaultAdapter;
}());

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * Cordova adapter for hybrid apps.
 */
var CordovaAdapter = (function () {
    function CordovaAdapter() {
    }
    CordovaAdapter.openBrowserTab = function (url, options) {
        var cordova = window.cordova;
        if (options.toolbarColor) {
            cordova.plugins.browsertab.themeable.openUrl(url, options);
        }
        else {
            cordova.plugins.browsertab.themeable.openUrl(url);
        }
        
    };
    CordovaAdapter.prototype.login = function (options) {
        //let promise = Keycloak.createPromise();
        var o = 'location=no';
        if (options && options.prompt === 'none') {
            o += ',hidden=yes';
        }
        var loginUrl = Keycloak.createLoginUrl(options);
        console.info('opening login frame from cordova: ' + loginUrl);
        if (!window.cordova) {
            throw new Error('Cannot authenticate via a web browser');
        }
        if (!window.cordova.InAppBrowser || !window.cordova.plugins.browsertab) {
            throw new Error('The Apache Cordova InAppBrowser/BrowserTab plugins was not found and are required');
        }
        var ref;
        //let ref = window.cordova.InAppBrowser.open(loginUrl, '_blank', o);
        //let ref = window.cordova.InAppBrowser.open(loginUrl, '_system', o);
        var completed = false;
        window.cordova.plugins.browsertab.themeable.isAvailable(function (result) {
            if (!result) {
                ref = window.cordova.InAppBrowser.open(loginUrl, '_system');
                ref.addEventListener('loadstart', function (event) {
                    if (event.url.indexOf('http://localhost') === 0) {
                        var callback = Keycloak.parseCallback(event.url);
                        Keycloak.processCallback(callback);
                        ref.close();
                        completed = true;
                    }
                });
                ref.addEventListener('loaderror', function (event) {
                    if (!completed) {
                        if (event.url.indexOf('http://localhost') === 0) {
                            var callback = Keycloak.parseCallback(event.url);
                            Keycloak.processCallback(callback);
                            ref.close();
                            completed = true;
                        }
                        else {
                            ref.close();
                        }
                    }
                });
            }
            else {
                CordovaAdapter.openBrowserTab(loginUrl, options);
            }
        }, function (isAvailableError) {
            console.info('failed to query availability of in-app browser tab');
        });
    };
    CordovaAdapter.prototype.closeBrowserTab = function () {
        var cordova = window.cordova;
        cordova.plugins.browsertab.themeable.close();
        //completed = true;
    };
    CordovaAdapter.prototype.logout = function (options) {
        var cordova = window.cordova;
        var logoutUrl = Keycloak.createLogoutUrl(options);
        var ref;
        var error;
        cordova.plugins.browsertab.themeable.isAvailable(function (result) {
            if (!result) {
                ref = cordova.InAppBrowser.open(logoutUrl, '_system');
                ref.addEventListener('loadstart', function (event) {
                    if (event.url.indexOf('http://localhost') === 0) {
                        this.ref.close();
                    }
                });
                ref.addEventListener('loaderror', function (event) {
                    if (event.url.indexOf('http://localhost') === 0) {
                        this.ref.close();
                    }
                    else {
                        error = true;
                        this.ref.close();
                    }
                });
                ref.addEventListener('exit', function (event) {
                    if (error) {
                        //promise.setError();
                    }
                    else {
                        Keycloak.clearToken({});
                        //promise.setSuccess();
                    }
                });
            }
            else {
                CordovaAdapter.openBrowserTab(logoutUrl, options);
            }
        }, function (isAvailableError) {
            console.info('failed to query availability of in-app browser tab');
        });
    };
    CordovaAdapter.prototype.register = function (options) {
        var registerUrl = Keycloak.createRegisterUrl({});
        window.cordova.plugins.browsertab.themeable.isAvailable(function (result) {
            if (!result) {
                window.cordova.InAppBrowser.open(registerUrl, '_system');
            }
            else {
                CordovaAdapter.openBrowserTab(registerUrl, options);
            }
        }, function (isAvailableError) {
            console.info('failed to query availability of in-app browser tab');
        });
    };
    CordovaAdapter.prototype.accountManagement = function (options) {
        var accountUrl = Keycloak.createAccountUrl({});
        window.cordova.plugins.browsertab.themeable.isAvailable(function (result) {
            if (!result) {
                window.cordova.InAppBrowser.open(accountUrl, '_system');
            }
            else {
                CordovaAdapter.openBrowserTab(accountUrl, options);
            }
        }, function (isAvailableError) {
            console.info('failed to query availability of in-app browser tab');
        });
    };
    CordovaAdapter.prototype.redirectUri = function (options) {
        if (options.redirectUri) {
            return options.redirectUri;
        }
        else {
            return 'http://localhost';
        }
    };
    return CordovaAdapter;
}());

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * To store Keycloak objects like tokens using a localStorage.
 */
var LocalStorage = (function () {
    function LocalStorage() {
    }
    LocalStorage.prototype.clearExpired = function () {
        var time = new Date().getTime();
        for (var i = 1; i <= localStorage.length; i++) {
            var key = localStorage.key(i);
            if (key && key.indexOf('kc-callback-') === 0) {
                var value = localStorage.getItem(key);
                if (value) {
                    try {
                        var expires = JSON.parse(value).expires;
                        if (!expires || expires < time) {
                            localStorage.removeItem(key);
                        }
                    }
                    catch (err) {
                        localStorage.removeItem(key);
                    }
                }
            }
        }
    };
    LocalStorage.prototype.get = function (state) {
        if (!state) {
            return;
        }
        var key = 'kc-callback-' + state;
        var value = localStorage.getItem(key);
        if (value) {
            localStorage.removeItem(key);
            value = JSON.parse(value);
        }
        this.clearExpired();
        return value;
    };
    
    LocalStorage.prototype.add = function (state) {
        this.clearExpired();
        var key = 'kc-callback-' + state.state;
        state.expires = new Date().getTime() + (60 * 60 * 1000);
        localStorage.setItem(key, JSON.stringify(state));
    };
    
    return LocalStorage;
}());

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * To store Keycloak objects like tokens using a cookie.
 */
var CookieStorage = (function () {
    function CookieStorage() {
        this.getCookie = function (key) {
            var name = key + '=';
            var ca = document.cookie.split(';');
            for (var i = 0; i < ca.length; i++) {
                var c = ca[i];
                while (c.charAt(0) === ' ') {
                    c = c.substring(1);
                }
                if (c.indexOf(name) === 0) {
                    return c.substring(name.length, c.length);
                }
            }
            return '';
        };
    }
    CookieStorage.prototype.get = function (state) {
        if (!state) {
            return;
        }
        var value = this.getCookie('kc-callback-' + state);
        this.setCookie('kc-callback-' + state, '', this.cookieExpiration(-100));
        if (value) {
            return JSON.parse(value);
        }
    };
    
    CookieStorage.prototype.add = function (state) {
        this.setCookie('kc-callback-' + state.state, JSON.stringify(state), this.cookieExpiration(60));
    };
    
    CookieStorage.prototype.removeItem = function (key) {
        this.setCookie(key, '', this.cookieExpiration(-100));
    };
    
    CookieStorage.prototype.cookieExpiration = function (minutes) {
        var exp = new Date();
        exp.setTime(exp.getTime() + (minutes * 60 * 1000));
        return exp;
    };
    
    CookieStorage.prototype.setCookie = function (key, value, expirationDate) {
        var cookie = key + '=' + value + '; '
            + 'expires=' + expirationDate.toUTCString() + '; ';
        document.cookie = cookie;
    };
    return CookieStorage;
}());

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * URI parser.
 */
var URIParser = (function () {
    function URIParser() {
    }
    URIParser.initialParse = function (uriToParse, responseMode) {
        var baseUri;
        var queryString;
        var fragmentString;
        var questionMarkIndex = uriToParse.indexOf('?');
        var fragmentIndex = uriToParse.indexOf('#', questionMarkIndex + 1);
        if (questionMarkIndex === -1 && fragmentIndex === -1) {
            baseUri = uriToParse;
        }
        else if (questionMarkIndex !== -1) {
            baseUri = uriToParse.substring(0, questionMarkIndex);
            queryString = uriToParse.substring(questionMarkIndex + 1);
            if (fragmentIndex !== -1) {
                fragmentIndex = queryString.indexOf('#');
                fragmentString = queryString.substring(fragmentIndex + 1);
                queryString = queryString.substring(0, fragmentIndex);
            }
        }
        else {
            baseUri = uriToParse.substring(0, fragmentIndex);
            fragmentString = uriToParse.substring(fragmentIndex + 1);
        }
        return { baseUri: baseUri, queryString: queryString, fragmentString: fragmentString };
    };
    URIParser.parseParams = function (paramString) {
        var result = {};
        var params = paramString.split('&');
        for (var i = 0; i < params.length; i++) {
            var p = params[i].split('=');
            var paramName = decodeURIComponent(p[0]);
            var paramValue = decodeURIComponent(p[1]);
            result[paramName] = paramValue;
        }
        return result;
    };
    URIParser.handleQueryParam = function (paramName, paramValue, oauth) {
        var supportedOAuthParams = ['code', 'state', 'error', 'error_description'];
        for (var i = 0; i < supportedOAuthParams.length; i++) {
            if (paramName === supportedOAuthParams[i]) {
                oauth[paramName] = paramValue;
                return true;
            }
        }
        return false;
    };
    URIParser.parseUri = function (uriToParse, responseMode) {
        var parsedUri = this.initialParse(decodeURIComponent(uriToParse), responseMode);
        var queryParams = {};
        if (parsedUri.queryString) {
            queryParams = this.parseParams(parsedUri.queryString);
        }
        var oauth = { newUrl: parsedUri.baseUri };
        for (var param in queryParams) {
            switch (param) {
                case 'redirect_fragment':
                    oauth.fragment = queryParams[param];
                    break;
                case 'prompt':
                    oauth.prompt = queryParams[param];
                    break;
                default:
                    if (responseMode !== 'query' || !this.handleQueryParam(param, queryParams[param], oauth)) {
                        oauth.newUrl += (oauth.newUrl.indexOf('?') === -1 ? '?' : '&') + param + '=' + queryParams[param];
                    }
                    break;
            }
        }
        if (responseMode === 'fragment') {
            var fragmentParams = {};
            if (parsedUri.fragmentString) {
                fragmentParams = this.parseParams(parsedUri.fragmentString);
            }
            for (var param in fragmentParams) {
                oauth[param] = fragmentParams[param];
            }
        }
        return oauth;
    };
    return URIParser;
}());

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * A login iframe.
 */
var LoginIframe = (function () {
    function LoginIframe(enable, callbackMap, interval) {
        this.enable = enable;
        this.callbackMap = callbackMap;
        this.interval = interval;
    }
    return LoginIframe;
}());

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * UUID generator.
 */
var UUID = (function () {
    function UUID() {
    }
    UUID.createUUID = function () {
        var s = [];
        var hexDigits = '0123456789abcdef';
        for (var i = 0; i < 36; i++) {
            s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
        }
        s[14] = '4';
        s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1);
        s[8] = s[13] = s[18] = s[23] = '-';
        var uuid = s.join('');
        return uuid;
    };
    return UUID;
}());

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * Token utility
 */
var Token = (function () {
    function Token() {
    }
    Token.decodeToken = function (str) {
        str = str.split('.')[1];
        str = str.replace('/-/g', '+');
        str = str.replace('/_/g', '/');
        switch (str.length % 4) {
            case 0:
                break;
            case 2:
                str += '==';
                break;
            case 3:
                str += '=';
                break;
            default:
                throw 'Invalid token';
        }
        str = (str + '===').slice(0, str.length + (str.length % 4));
        str = str.replace(/-/g, '+').replace(/_/g, '/');
        str = decodeURIComponent(atob(str));
        str = JSON.parse(str);
        return str;
    };
    return Token;
}());

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
function teste2() {
    var observer = new Array();
    if (Keycloak.loginIframe.iframe && Keycloak.loginIframe.iframeOrigin) {
        var msg = Keycloak.clientId + ' ' + Keycloak.sessionId;
        // Keycloak.loginIframe.callbackMap[Keycloak.clientId] = observer;
        var origin = Keycloak.loginIframe.iframeOrigin;
        Keycloak.loginIframe.iframe.contentWindow.postMessage(msg, origin);
        observer.push(true);
    }
    else {
        // promise.setSuccess();
        observer.push(false);
    }
    return rxjs_Rx.Observable.from(observer);
}
function teste(minValidity) {
    //let observer:Observer<string> = Observable.create();
    var observer = new Array();
    minValidity = minValidity || 5;
    if (!Keycloak.isTokenExpired(minValidity)) {
        console.info('token still valid');
        //observer.next(Keycloak.accessToken);
        observer.push(Keycloak.accessToken);
    }
    else {
        if (Keycloak.isRefreshTokenExpired(5)) {
            Keycloak.login(Keycloak.config);
        }
        else {
            console.info('refreshing token');
            var params = new _angular_http.URLSearchParams();
            params.set('grant_type', 'refresh_token');
            params.set('refresh_token', Keycloak.refreshToken);
            var url = Keycloak.getRealmUrl() + '/protocol/openid-connect/token';
            console.info('getting url');
            var headers = new _angular_http.Headers({ 'Content-type': 'application/x-www-form-urlencoded' });
            if (Keycloak.clientId && Keycloak.clientSecret) {
                headers.append('Authorization', 'Basic ' + btoa(Keycloak.clientId + ': ' + Keycloak.clientSecret));
            }
            else {
                params.set('client_id', Keycloak.clientId);
            }
            var timeLocal_1 = new Date().getTime();
            var options = { headers: headers, withCredentials: true };
            console.info('calling url ' + url);
            this.http.post(url, params, options).subscribe(function (token) {
                timeLocal_1 = (timeLocal_1 + new Date().getTime()) / 2;
                var tokenResponse = token.json();
                console.info('parsed access token ' + tokenResponse['access_token']);
                Keycloak.setToken(tokenResponse['access_token'], tokenResponse['refresh_token'], tokenResponse['id_token'], true);
                Keycloak.timeSkew = Math.floor(timeLocal_1 / 1000) - Keycloak.tokenParsed.iat;
                observer.push(tokenResponse['access_token']);
            });
        }
    }
    return rxjs_Rx.Observable.from(observer);
}
var Keycloak = (function () {
    // public constructor
    function Keycloak() {
        Keycloak_1.loginIframe = new LoginIframe(true, [], 5);
    }
    Keycloak_1 = Keycloak;
    // Keycloak methods
    Keycloak.login = function (options) {
        return Keycloak_1.adapter.login(options);
    };
    Keycloak.logout = function (options) {
        return Keycloak_1.adapter.logout(options);
    };
    Keycloak.updateToken = function (minValidity) {
        return teste(minValidity);
        /*return new Observable<string>((observer:Observer<string>) => {

            minValidity = minValidity || 5;

            if (!Keycloak.isTokenExpired(minValidity)) {
                console.info('token still valid');
                observer.next(Keycloak.accessToken);
            } else {
                if (Keycloak.isRefreshTokenExpired(5)) {
                    Keycloak.login(Keycloak.config);
                } else {
                    console.info('refreshing token');
                    let params:URLSearchParams = new URLSearchParams();
                    params.set('grant_type', 'refresh_token');
                    params.set('refresh_token', Keycloak.refreshToken);

                    let url = Keycloak.getRealmUrl() + '/protocol/openid-connect/token';
                    console.info('getting url');
                    let headers = new Headers({'Content-type': 'application/x-www-form-urlencoded'});

                    if (Keycloak.clientId && Keycloak.clientSecret) {
                        headers.append('Authorization', 'Basic ' + btoa(Keycloak.clientId + ': ' + Keycloak.clientSecret));
                    } else {
                        params.set('client_id', Keycloak.clientId);
                    }

                    let timeLocal = new Date().getTime();

                    let options:RequestOptionsArgs = {headers: headers, withCredentials: true};
                    console.info('calling url ' + url);
                    this.http.post(url, params, options).subscribe((token:any) => {
                        timeLocal = (timeLocal + new Date().getTime()) / 2;

                        let tokenResponse = token.json();
                        console.info('parsed access token ' + tokenResponse['access_token']);
                        Keycloak.setToken(tokenResponse['access_token'], tokenResponse['refresh_token'], tokenResponse['id_token'], true);

                        Keycloak.timeSkew = Math.floor(timeLocal / 1000) - Keycloak.tokenParsed.iat;
                        observer.next(tokenResponse['access_token']);
                    });
                }
            }
        });*/
    };
    Keycloak.register = function (options) {
        return Keycloak_1.adapter.register(options);
    };
    Keycloak.accountManagement = function () {
        return Keycloak_1.adapter.accountManagement();
    };
    Keycloak.loadUserProfile = function () {
        var url = Keycloak_1.getRealmUrl() + '/account';
        var headers = new _angular_http.Headers({ 'Accept': 'application/json', 'Authorization': 'bearer ' + Keycloak_1.accessToken });
        var options = { headers: headers };
        return Keycloak_1.http.get(url, options).map(function (profile) { return profile.json(); });
    };
    Keycloak.loadUserInfo = function () {
        var url = Keycloak_1.getRealmUrl() + '/protocol/openid-connect/userinfo';
        var headers = new _angular_http.Headers({ 'Accept': 'application/json', 'Authorization': 'bearer ' + Keycloak_1.accessToken });
        var options = { headers: headers };
        return Keycloak_1.http.get(url, options).map(function (profile) { return profile; });
    };
    Keycloak.hasRealmRole = function (role) {
        var access = Keycloak_1.realmAccess;
        return !!access && access.roles.indexOf(role) >= 0;
    };
    Keycloak.hasResourceRole = function (role, resource) {
        if (!Keycloak_1.resourceAccess) {
            return false;
        }
        var access = Keycloak_1.resourceAccess[resource || Keycloak_1.clientId];
        return !!access && access.roles.indexOf(role) >= 0;
    };
    Keycloak.isTokenExpired = function (minValidity) {
        if (!Keycloak_1.tokenParsed || (!Keycloak_1.refreshToken && Keycloak_1.flow !== 'implicit')) {
            throw 'Not authenticated';
        }
        var expiresIn = Keycloak_1.tokenParsed['exp'] - (new Date().getTime() / 1000) + Keycloak_1.timeSkew;
        if (minValidity) {
            expiresIn -= minValidity;
        }
        return expiresIn < 0;
    };
    Keycloak.isRefreshTokenExpired = function (minValidity) {
        if (!Keycloak_1.tokenParsed || (!Keycloak_1.refreshToken && Keycloak_1.flow !== 'implicit')) {
            throw 'Not authenticated';
        }
        var expiresIn = Keycloak_1.refreshTokenParsed['exp'] - (new Date().getTime() / 1000) + Keycloak_1.timeSkew;
        if (minValidity) {
            expiresIn -= minValidity;
        }
        return expiresIn < 0;
    };
    Keycloak.clearToken = function (initOptions) {
        if (Keycloak_1.accessToken) {
            Keycloak_1.setToken(null, null, null, true);
            Keycloak_1.authenticatedBehaviourSubject.next(false);
            if (Keycloak_1.loginRequired) {
                Keycloak_1.login(initOptions);
            }
        }
    };
    // URLs methods
    Keycloak.createLoginUrl = function (options) {
        var state = UUID.createUUID();
        var nonce = UUID.createUUID();
        var redirectUri = Keycloak_1.adapter.redirectUri(options);
        if (options && options.prompt) {
            redirectUri += (redirectUri.indexOf('?') === -1 ? '?' : '&') + 'prompt=' + options.prompt;
        }
        Keycloak_1.callbackStorage.add({ state: state, nonce: nonce, redirectUri: redirectUri });
        var action = 'auth';
        if (options && options.action === 'register') {
            action = 'registrations';
        }
        var scope = (options && options.scope) ? 'openid ' + options.scope : 'openid';
        var url = Keycloak_1.getRealmUrl()
            + '/protocol/openid-connect/' + action
            + '?client_id=' + encodeURIComponent(Keycloak_1.clientId)
            + '&redirect_uri=' + encodeURIComponent(redirectUri)
            + '&state=' + encodeURIComponent(state)
            + '&nonce=' + encodeURIComponent(nonce)
            + '&response_mode=' + encodeURIComponent(Keycloak_1.responseMode)
            + '&response_type=' + encodeURIComponent(Keycloak_1.responseType)
            + '&scope=' + encodeURIComponent(scope);
        if (options && options.prompt) {
            url += '&prompt=' + encodeURIComponent(options.prompt);
        }
        if (options && options.maxAge) {
            url += '&max_age=' + encodeURIComponent(options.maxAge);
        }
        if (options && options.loginHint) {
            url += '&login_hint=' + encodeURIComponent(options.loginHint);
        }
        if (options && options.idpHint) {
            url += '&kc_idp_hint=' + encodeURIComponent(options.idpHint);
        }
        if (options && options.locale) {
            url += '&ui_locales=' + encodeURIComponent(options.locale);
        }
        return url;
    };
    Keycloak.createLogoutUrl = function (options) {
        var url = Keycloak_1.getRealmUrl()
            + '/protocol/openid-connect/logout'
            + '?redirect_uri=' + encodeURIComponent(Keycloak_1.adapter.redirectUri(options, false));
        return url;
    };
    Keycloak.createRegisterUrl = function (options) {
        if (!options) {
            options = {};
        }
        options.action = 'register';
        return Keycloak_1.createLoginUrl(options);
    };
    Keycloak.createAccountUrl = function (options) {
        var url = Keycloak_1.getRealmUrl()
            + '/account'
            + '?referrer=' + encodeURIComponent(Keycloak_1.clientId)
            + '&referrer_uri=' + encodeURIComponent(Keycloak_1.adapter.redirectUri(options));
        return url;
    };
    Keycloak.getRealmUrl = function () {
        if (Keycloak_1.authServerUrl.charAt(Keycloak_1.authServerUrl.length - 1) === '/') {
            return Keycloak_1.authServerUrl + 'realms/' + encodeURIComponent(Keycloak_1.realm);
        }
        else {
            return Keycloak_1.authServerUrl + '/realms/' + encodeURIComponent(Keycloak_1.realm);
        }
    };
    Keycloak.checkLoginIframe = function () {
        return teste2();
        /*return new Observable<boolean>((observer: any) => {
            if (Keycloak.loginIframe.iframe && Keycloak.loginIframe.iframeOrigin) {

                let msg = Keycloak.clientId + ' ' + Keycloak.sessionId;
                // Keycloak.loginIframe.callbackMap[Keycloak.clientId] = observer;
                let origin = Keycloak.loginIframe.iframeOrigin;
                Keycloak.loginIframe.iframe.contentWindow.postMessage(msg, origin);
                observer.next(true);
            } else {
                // promise.setSuccess();
                observer.next(false);
            }
        });*/
    };
    Keycloak.processCallback = function (oauth) {
        var code = oauth.code;
        var error = oauth.error;
        var prompt = oauth.prompt;
        var timeLocal = new Date().getTime();
        if (error) {
            if (prompt !== 'none') {
                var errorData = { error: error, error_description: oauth.error_description };
                Keycloak_1.authErrorBehaviourSubject.next(errorData);
            }
            return;
        }
        else if ((Keycloak_1.flow !== 'standard') && (oauth.access_token || oauth.id_token)) {
            authSuccess(oauth.access_token, null, oauth.id_token, true);
        }
        if ((Keycloak_1.flow !== 'implicit') && code) {
            var url = Keycloak_1.getRealmUrl() + '/protocol/openid-connect/token';
            var params = new _angular_http.URLSearchParams();
            params.set('code', code);
            params.set('grant_type', 'authorization_code');
            var headers = new _angular_http.Headers({ 'Content-type': 'application/x-www-form-urlencoded' });
            if (Keycloak_1.clientId && Keycloak_1.clientSecret) {
                headers.append('Authorization', 'Basic ' + btoa(Keycloak_1.clientId + ':' + Keycloak_1.clientSecret));
            }
            else {
                params.set('client_id', Keycloak_1.clientId);
            }
            params.set('redirect_uri', oauth.redirectUri);
            var options = { headers: headers, withCredentials: true };
            this.http.post(url, params, options).subscribe(function (token) {
                var tokenResponse = token.json();
                authSuccess(tokenResponse['access_token'], tokenResponse['refresh_token'], tokenResponse['id_token'], Keycloak_1.flow === 'standard');
            });
        }
        function authSuccess(accessToken, refreshToken, idToken, fulfillPromise) {
            timeLocal = (timeLocal + new Date().getTime()) / 2;
            Keycloak_1.setToken(accessToken, refreshToken, idToken, true);
            if ((Keycloak_1.tokenParsed && Keycloak_1.tokenParsed.nonce !== oauth.storedNonce) ||
                (Keycloak_1.refreshTokenParsed && Keycloak_1.refreshTokenParsed.nonce !== oauth.storedNonce) ||
                (Keycloak_1.idTokenParsed && Keycloak_1.idTokenParsed.nonce !== oauth.storedNonce)) {
                console.log('invalid nonce!');
                Keycloak_1.clearToken({});
            }
            else {
                Keycloak_1.timeSkew = Math.floor(timeLocal / 1000) - Keycloak_1.tokenParsed.iat;
                if (fulfillPromise) {
                    Keycloak_1.authSuccessBehaviourSubject.next(true);
                }
            }
        }
    };
    Keycloak.setToken = function (accessToken, refreshToken, idToken, useTokenTime) {
        if (Keycloak_1.tokenTimeoutHandle) {
            clearTimeout(Keycloak_1.tokenTimeoutHandle);
            Keycloak_1.tokenTimeoutHandle = null;
        }
        if (accessToken) {
            Keycloak_1.accessToken = accessToken;
            Keycloak_1.tokenParsed = Token.decodeToken(accessToken);
            var sessionId = Keycloak_1.realm + '/' + Keycloak_1.tokenParsed.sub;
            if (Keycloak_1.tokenParsed.session_state) {
                sessionId = sessionId + '/' + Keycloak_1.tokenParsed.session_state;
            }
            Keycloak_1.sessionId = sessionId;
            Keycloak_1.authenticatedBehaviourSubject.next(true);
            Keycloak_1.subject = Keycloak_1.tokenParsed.sub;
            Keycloak_1.realmAccess = Keycloak_1.tokenParsed.realm_access;
            Keycloak_1.resourceAccess = Keycloak_1.tokenParsed.resource_access;
            var start = useTokenTime ? Keycloak_1.tokenParsed.iat : (new Date().getTime() / 1000);
            var expiresIn = Keycloak_1.tokenParsed.exp - start;
            Keycloak_1.tokenTimeoutHandle = setTimeout(Keycloak_1.tokenExpiredBehaviourSubject.next(true), expiresIn * 1000);
        }
        else {
            delete Keycloak_1.accessToken;
            delete Keycloak_1.tokenParsed;
            delete Keycloak_1.subject;
            delete Keycloak_1.realmAccess;
            delete Keycloak_1.resourceAccess;
        }
        if (refreshToken) {
            Keycloak_1.refreshToken = refreshToken;
            Keycloak_1.refreshTokenParsed = Token.decodeToken(refreshToken);
        }
        else {
            delete Keycloak_1.refreshToken;
            delete Keycloak_1.refreshTokenParsed;
        }
        if (idToken) {
            Keycloak_1.idToken = idToken;
            Keycloak_1.idTokenParsed = Token.decodeToken(idToken);
        }
        else {
            delete Keycloak_1.idToken;
            delete Keycloak_1.idTokenParsed;
        }
    };
    Keycloak.createCallbackId = function () {
        var id = '<id: ' + (Keycloak_1.callback_id++) + (Math.random()) + '>';
        return id;
    };
    Keycloak.parseCallback = function (url) {
        var oauth = URIParser.parseUri(url, Keycloak_1.responseMode);
        var state = oauth.state;
        var oauthState = Keycloak_1.callbackStorage.get(state);
        if (oauthState && (oauth.code || oauth.error || oauth.access_token || oauth.id_token)) {
            oauth.redirectUri = oauthState.redirectUri;
            oauth.storedNonce = oauthState.nonce;
            if (oauth.fragment) {
                oauth.newUrl += '#' + oauth.fragment;
            }
            return oauth;
        }
    };
    Keycloak.prototype.init = function (initOptions) {
        // should use a better lock
        var _this = this;
        if (!Keycloak_1.initializedBehaviourSubject.getValue()) {
            console.info('KC_CORE: initializing...');
            try {
                Keycloak_1.callbackStorage = new LocalStorage();
            }
            catch (err) {
                Keycloak_1.callbackStorage = new CookieStorage();
            }
            if (initOptions && initOptions.adapter === 'cordova') {
                Keycloak_1.adapter = this.loadAdapter('cordova');
            }
            else if (initOptions && initOptions.adapter === 'default') {
                Keycloak_1.adapter = this.loadAdapter('default');
            }
            else {
                if (window['cordova']) {
                    Keycloak_1.adapter = this.loadAdapter('cordova');
                }
                else {
                    Keycloak_1.adapter = this.loadAdapter('default');
                }
            }
            // options processing
            if (initOptions) {
                if (typeof initOptions.checkLoginIframe !== 'undefined') {
                    Keycloak_1.loginIframe.enable = initOptions.checkLoginIframe;
                }
                if (initOptions.checkLoginIframeInterval) {
                    Keycloak_1.loginIframe.interval = initOptions.checkLoginIframeInterval;
                }
                if (initOptions.onLoad === 'login-required') {
                    Keycloak_1.loginRequired = true;
                }
                if (initOptions.responseMode) {
                    if (initOptions.responseMode === 'query' || initOptions.responseMode === 'fragment') {
                        Keycloak_1.responseMode = initOptions.responseMode;
                    }
                    else {
                        throw 'Invalid value for responseMode';
                    }
                }
                if (initOptions.flow) {
                    switch (initOptions.flow) {
                        case 'standard':
                            Keycloak_1.responseType = 'code';
                            break;
                        case 'implicit':
                            Keycloak_1.responseType = 'id_token token';
                            break;
                        case 'hybrid':
                            Keycloak_1.responseType = 'code id_token token';
                            break;
                        default:
                            throw 'Invalid value for flow';
                    }
                    Keycloak_1.flow = initOptions.flow;
                }
            }
            if (!Keycloak_1.responseMode) {
                Keycloak_1.responseMode = 'fragment';
            }
            if (!Keycloak_1.responseType) {
                Keycloak_1.responseType = 'code';
                Keycloak_1.flow = 'standard';
            }
            // loading keycloak conf
            this.loadConfig(Keycloak_1.config).subscribe(function (loaded) {
                if (loaded) {
                    _this.processInit(initOptions).subscribe(function (initialized) {
                        Keycloak_1.initializedBehaviourSubject.next(true);
                    });
                }
            });
        }
    };
    Keycloak.prototype.loadConfig = function (url) {
        return new rxjs_Rx.Observable(function (observer) {
            var configUrl;
            if (!Keycloak_1.config) {
                configUrl = 'keycloak.json';
            }
            else if (typeof Keycloak_1.config === 'string') {
                configUrl = Keycloak_1.config;
            }
            if (configUrl) {
                Keycloak_1.http.get(configUrl).map(function (res) { return res.json(); }).subscribe(function (config) {
                    Keycloak_1.authServerUrl = config['auth-server-url'];
                    Keycloak_1.realm = config['realm'];
                    Keycloak_1.clientId = config['resource'];
                    Keycloak_1.clientSecret = (config['credentials'] || {})['secret'];
                    observer.next(true);
                    // console.info("Keycloak initialized !");
                });
            }
            else {
                if (!Keycloak_1.config['url']) {
                    var scripts = document.getElementsByTagName('script');
                    for (var i = 0; i < scripts.length; i++) {
                        if (scripts[i].src.match(/.*keycloak\.js/)) {
                            Keycloak_1.config.url = scripts[i].src.substr(0, scripts[i].src.indexOf('/js/keycloak.js'));
                            break;
                        }
                    }
                }
                if (!Keycloak_1.config.realm) {
                    throw 'realm missing';
                }
                if (!Keycloak_1.config.clientId) {
                    throw 'clientId missing';
                }
                Keycloak_1.authServerUrl = Keycloak_1.config.url;
                Keycloak_1.realm = Keycloak_1.config.realm;
                Keycloak_1.clientId = Keycloak_1.config.clientId;
                Keycloak_1.clientSecret = (Keycloak_1.config.credentials || {}).secret;
                observer.next(true);
            }
        });
    };
    Keycloak.prototype.processInit = function (initOptions) {
        var _this = this;
        return new rxjs_Rx.Observable(function (observer) {
            var callback = Keycloak_1.parseCallback(window.location.href);
            // let initPromise:any = Keycloak.createPromise;
            if (callback) {
                _this.setupCheckLoginIframe().subscribe(function (setup) {
                    window.history.replaceState({}, null, callback.newUrl);
                    Keycloak_1.processCallback(callback);
                });
            }
            else if (initOptions) {
                if (initOptions.token || initOptions.refreshToken) {
                    Keycloak_1.setToken(initOptions.token, initOptions.refreshToken, initOptions.idToken, false);
                    Keycloak_1.timeSkew = initOptions.timeSkew || 0;
                    if (Keycloak_1.loginIframe.enable) {
                        _this.setupCheckLoginIframe().subscribe(function (setup) {
                            Keycloak_1.checkLoginIframe().subscribe(function (checked) {
                                observer.next(true);
                            });
                        });
                    }
                    else {
                        observer.next(true);
                    }
                }
                else if (initOptions.onLoad) {
                    var options_1 = {};
                    var doLogin_1 = function (prompt) {
                        if (!prompt) {
                            options_1.prompt = 'none';
                        }
                        Keycloak_1.login(options_1);
                    };
                    switch (initOptions.onLoad) {
                        case 'check-sso':
                            console.info('login iframe ? ' + Keycloak_1.loginIframe.enable);
                            if (Keycloak_1.loginIframe.enable) {
                                _this.setupCheckLoginIframe().subscribe(function (setup) {
                                    Keycloak_1.checkLoginIframe().subscribe(function (checked) {
                                        doLogin_1(false);
                                    });
                                });
                            }
                            else {
                                doLogin_1(false);
                            }
                            break;
                        case 'login-required':
                            doLogin_1(true);
                            break;
                        default:
                            throw 'Invalid value for onLoad';
                    }
                }
                else {
                    observer.next(true);
                }
            }
            else {
                observer.next(true);
            }
        });
    };
    Keycloak.prototype.setupCheckLoginIframe = function () {
        return new rxjs_Rx.Observable(function (observer) {
            console.info('setting up login iframe...');
            if (!Keycloak_1.loginIframe.enable) {
                console.info('setting up login iframe ended 1');
                observer.next(true);
                return;
            }
            if (Keycloak_1.loginIframe.iframe) {
                console.info('setting up login iframe ended 2');
                observer.next(true);
            }
            var iframe = document.createElement('iframe');
            Keycloak_1.loginIframe.iframe = iframe;
            var check = function () {
                Keycloak_1.checkLoginIframe().subscribe(function (check) {
                    console.info('iframe checked');
                });
                if (Keycloak_1.accessToken) {
                    setTimeout(check, Keycloak_1.loginIframe.interval * 1000);
                }
            };
            iframe.onload = function () {
                var realmUrl = Keycloak_1.getRealmUrl();
                if (realmUrl.charAt(0) === '/') {
                    var origin = void 0;
                    if (!window.location.origin) {
                        origin = window.location.protocol
                            + '//' + window.location.hostname
                            + (window.location.port ? ': ' + window.location.port : '');
                    }
                    else {
                        origin = window.location.origin;
                    }
                    Keycloak_1.loginIframe.iframeOrigin = origin;
                }
                else {
                    Keycloak_1.loginIframe.iframeOrigin = realmUrl.substring(0, realmUrl.indexOf('/', 8));
                }
                console.info('setting up login iframe ended 3');
                observer.next(true);
                setTimeout(check, Keycloak_1.loginIframe.interval * 1000);
            };
            var src = Keycloak_1.getRealmUrl() + '/protocol/openid-connect/login-status-iframe.html';
            console.info('reloading iframe...' + src);
            iframe.setAttribute('src', src);
            iframe.style.display = 'none';
            document.body.appendChild(iframe);
            var messageCallback = function (event) {
                console.info('checking iframe message callback..' + event.data);
                if (event.origin !== iframe.iframeOrigin) {
                    console.info('setting up login iframe ended 7');
                }
                if (event.origin === iframe.iframeOrigin && event.data !== 'unchanged') {
                    console.info('setting up login iframe ended 6');
                    Keycloak_1.clearToken({});
                }
            };
            window.addEventListener('message', messageCallback, false);
        });
    };
    Keycloak.prototype.loadAdapter = function (type) {
        if (!type || type === 'default') {
            return new DefaultAdapter();
        }
        if (type === 'cordova') {
            return new CordovaAdapter();
        }
        throw 'invalid adapter type: ' + type;
    };
    // internal objects
    Keycloak.config = undefined;
    Keycloak.adapter = undefined;
    Keycloak.resourceAccess = undefined;
    Keycloak.callback_id = 0;
    Keycloak.loginIframe = undefined;
    Keycloak.callbackStorage = undefined;
    Keycloak.timeSkew = undefined;
    Keycloak.http = undefined;
    Keycloak.loginRequired = undefined;
    // Token objects
    Keycloak.tokenParsed = undefined;
    Keycloak.idTokenParsed = undefined;
    Keycloak.refreshTokenParsed = undefined;
    Keycloak.accessToken = undefined;
    Keycloak.idToken = undefined;
    Keycloak.refreshToken = undefined;
    Keycloak.tokenTimeoutHandle = undefined;
    Keycloak.subject = undefined;
    Keycloak.sessionId = undefined;
    // OIDC client properties
    Keycloak.responseMode = undefined;
    Keycloak.responseType = undefined;
    Keycloak.flow = undefined;
    Keycloak.clientId = undefined;
    Keycloak.clientSecret = undefined;
    Keycloak.authServerUrl = undefined;
    Keycloak.realm = undefined;
    Keycloak.realmAccess = undefined;
    // Keycloak state subjects
    Keycloak.initializedBehaviourSubject = new rxjs_Rx.BehaviorSubject(false);
    Keycloak.authenticatedBehaviourSubject = new rxjs_Rx.BehaviorSubject(false);
    Keycloak.authSuccessBehaviourSubject = new rxjs_Rx.BehaviorSubject(false);
    Keycloak.authErrorBehaviourSubject = new rxjs_Rx.BehaviorSubject({});
    Keycloak.tokenExpiredBehaviourSubject = new rxjs_Rx.BehaviorSubject(false);
    // Keycloak state observables
    Keycloak.initializedObs = Keycloak_1.initializedBehaviourSubject.asObservable();
    Keycloak.authenticatedObs = Keycloak_1.authenticatedBehaviourSubject.asObservable();
    Keycloak.authSuccessObs = Keycloak_1.authSuccessBehaviourSubject.asObservable();
    Keycloak.authErrorObs = Keycloak_1.authErrorBehaviourSubject.asObservable();
    Keycloak.tokenExpiredObs = Keycloak_1.tokenExpiredBehaviourSubject.asObservable();
    Keycloak = Keycloak_1 = __decorate([
        _angular_core.Injectable()
    ], Keycloak);
    return Keycloak;
    var Keycloak_1;
}());

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate$2 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/**
 * Keycloak Authorization manager.
 *
 * Manager authorization headers and tokens to access to protected resources.
 */
function teste$1(resourceSeververId) {
    var _this = this;
    var observer = new Array();
    var url = Keycloak.authServerUrl + '/realms/' + Keycloak.realm + '/authz/entitlement/' + resourceSeververId;
    var headers = new _angular_http.Headers({ 'Authorization': 'Bearer ' + Keycloak.accessToken });
    var options = { headers: headers };
    Keycloak.http.get(url, options).map(function (token) {
        var status = token.status;
        if (status >= 200 && status < 300) {
            var rpt = JSON.parse(token.text()).rpt;
            _this.rpt = rpt;
            observer.push(rpt);
        }
        else if (status === 403) {
            console.error('Authorization request was denied by the server.');
            rxjs_Rx.Observable.throw('Authorization request was denied by the server.');
        }
        else {
            console.error('Could not obtain authorization data from server.');
            rxjs_Rx.Observable.throw('Authorization request was denied by the server.');
        }
    });
    return rxjs_Rx.Observable.from(observer);
}
var KeycloakAuthorization = (function () {
    // constructor
    function KeycloakAuthorization(keycloak) {
        this.keycloak = keycloak;
    }
    KeycloakAuthorization_1 = KeycloakAuthorization;
    /**
     * This method enables client applications to better integrate with resource servers protected by a Keycloak
     * policy enforcer.
     *
     * In this case, the resource server will respond with a 401 status code and a WWW-Authenticate header holding the
     * necessary information to ask a Keycloak server for authorization data using both UMA and Entitlement protocol,
     * depending on how the policy enforcer at the resource server was configured.
     */
    KeycloakAuthorization.authorize = function (wwwAuthenticateHeader) {
        var _this = this;
        if (wwwAuthenticateHeader.indexOf('UMA') !== -1) {
            var params = wwwAuthenticateHeader.split(',');
            var headers = void 0;
            var body = void 0;
            for (var i = 0; i < params.length; i++) {
                var param = params[i].split('=');
                if (param[0] === 'ticket') {
                    var ticket = param[1].substring(1, param[1].length - 1).trim();
                    headers = new _angular_http.Headers({ 'Content-type': 'application/json' });
                    headers.append('Authorization', 'Bearer ' + Keycloak.accessToken);
                    body = JSON.stringify({
                        ticket: ticket,
                        rpt: this.rpt
                    });
                }
            }
            var options = { headers: headers };
            return Keycloak.http.post(KeycloakAuthorization_1.config.rpt_endpoint, body, options).map(function (token) {
                var status = token.status;
                if (status >= 200 && status < 300) {
                    // Token retrieved
                    var rpt = JSON.parse(token.text()).rpt;
                    _this.rpt = rpt;
                    return rpt;
                }
                else if (status === 403) {
                    console.error('Authorization request was denied by the server.');
                    rxjs_Rx.Observable.throw('Authorization request was denied by the server.');
                }
                else {
                    console.error('Could not obtain authorization data from server.');
                    rxjs_Rx.Observable.throw('Could not obtain authorization data from server.');
                }
            });
        }
        else if (wwwAuthenticateHeader.indexOf('KC_ETT') !== -1) {
            var params = wwwAuthenticateHeader.substring('KC_ETT'.length).trim().split(',');
            var clientId = null;
            for (var i = 0; i < params.length; i++) {
                var param = params[i].split('=');
                if (param[0] === 'realm') {
                    clientId = param[1].substring(1, param[1].length - 1).trim();
                }
            }
            return this.entitlement(clientId);
        }
    };
    /**
     * Obtains all entitlements from a Keycloak Server based on a give resourceServerId.
     */
    KeycloakAuthorization.entitlement = function (resourceSeververId) {
        return teste$1(resourceSeververId);
        /*return new Observable<any>((observer: any) => {

            let url = Keycloak.authServerUrl + '/realms/' + Keycloak.realm + '/authz/entitlement/' + resourceSeververId;
            let headers = new Headers({'Authorization': 'Bearer ' + Keycloak.accessToken});
            let options: RequestOptionsArgs = {headers: headers};

            Keycloak.http.get(url, options).map(token => {
                let status = token.status;

                if (status >= 200 && status < 300) {
                    let rpt: any = JSON.parse(token.text()).rpt;
                    this.rpt = rpt;

                    observer.next(rpt);
                } else if (status === 403) {
                    console.error('Authorization request was denied by the server.');
                    Observable.throw('Authorization request was denied by the server.');
                } else {
                    console.error('Could not obtain authorization data from server.');
                    Observable.throw('Authorization request was denied by the server.');
                }
            });
        });*/
    };
    
    KeycloakAuthorization.prototype.init = function () {
        if (!KeycloakAuthorization_1.initializedBehaviourSubject.getValue()) {
            Keycloak.initializedObs.filter(function (status) { return status === true; }).take(1).subscribe(function (status) {
                console.info('KC_AUTHZ: Keycloak initialized, loading authz...');
                var url = Keycloak.authServerUrl + '/realms/' + Keycloak.realm + '/.well-known/uma-configuration';
                var headers = new _angular_http.Headers({ 'Accept': 'application/json' });
                var options = { headers: headers };
                Keycloak.http.get(url, options).subscribe(function (authz) {
                    KeycloakAuthorization_1.config = authz.json();
                    // notifying initialization
                    KeycloakAuthorization_1.initializedBehaviourSubject.next(true);
                });
            });
        }
        this.keycloak.init({});
    };
    
    KeycloakAuthorization.config = undefined;
    KeycloakAuthorization.initializedBehaviourSubject = new rxjs_Rx.BehaviorSubject(false);
    KeycloakAuthorization.initializedObs = KeycloakAuthorization_1.initializedBehaviourSubject.asObservable();
    KeycloakAuthorization.rpt = undefined;
    KeycloakAuthorization = KeycloakAuthorization_1 = __decorate$2([
        _angular_core.Injectable()
    ], KeycloakAuthorization);
    return KeycloakAuthorization;
    var KeycloakAuthorization_1;
}());

/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$1 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/**
 * An Angular http proxy supporting Keycloak auth & authz.
 * Authenticate user, manage tokens and add authorization header to access to remote Keycloak protected resources.
 */
var KeycloakHttp = (function (_super) {
    __extends(KeycloakHttp, _super);
    function KeycloakHttp(backend, defaultOptions, keycloak, keycloakAuth) {
        var _this = _super.call(this, backend, defaultOptions) || this;
        _this.keycloak = keycloak;
        _this.keycloakAuth = keycloakAuth;
        _this.MAX_UNAUTHORIZED_ATTEMPTS = 2;
        Keycloak.http = new _angular_http.Http(backend, defaultOptions);
        _this.keycloak.init({});
        _this.keycloakAuth.init();
        return _this;
    }
    KeycloakHttp_1 = KeycloakHttp;
    KeycloakHttp.prototype.get = function (url, options) {
        // console.info("GET");
        options = options || { withCredentials: false };
        options.method = _angular_http.RequestMethod.Get;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.post = function (url, body, options) {
        options = options || { withCredentials: false };
        options.method = _angular_http.RequestMethod.Post;
        options.body = body;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.put = function (url, body, options) {
        options = options || { withCredentials: false };
        options.method = _angular_http.RequestMethod.Put;
        options.body = body;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.delete = function (url, options) {
        options = options || { withCredentials: false };
        options.method = _angular_http.RequestMethod.Delete;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.patch = function (url, body, options) {
        options = options || { withCredentials: false };
        options.method = _angular_http.RequestMethod.Patch;
        options.body = body;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.head = function (url, options) {
        options = options || { withCredentials: false };
        options.method = _angular_http.RequestMethod.Head;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.configureRequest = function (url, count, options) {
        var _this = this;
        if (options.withCredentials && !KeycloakHttp_1.readyBehaviourSubject.getValue()) {
            KeycloakAuthorization.initializedObs.take(1).filter(function (init) { return init === true; }).subscribe(function () {
                console.info('KC_HTTP: keycloak authz initialized...');
            });
            Keycloak.initializedObs.take(1).filter(function (init) { return init === true; }).subscribe(function () {
                if (!Keycloak.authenticatedBehaviourSubject.getValue()) {
                    console.info('KC_HTTP: keycloak initialized, go login...');
                    Keycloak.login(true);
                }
            });
            Keycloak.authenticatedObs.take(2).filter(function (auth) { return auth === true; }).subscribe(function () {
                console.info('KC_HTTP: authentication done...');
                KeycloakHttp_1.readyBehaviourSubject.next(true);
            });
            return KeycloakHttp_1.readyObs.take(2).filter(function (ready) { return ready === true; }).flatMap(function (ready) {
                console.info('KC_HTTP: keycloak is now http ready, re-attempting request...');
                return _this.configureRequest(url, count, options);
            });
        }
        else {
            // KC is ready, getting authorization header
            return this.setHeaders(options).flatMap(function (options) {
                console.info('KC_HTTP: using headers ' + options);
                // calling http with headers
                return _super.prototype.request.call(_this, url, options).catch(function (error) {
                    // error handling
                    var status = error.status;
                    if ((status === 403 || status === 401) && count < _this.MAX_UNAUTHORIZED_ATTEMPTS) {
                        console.info('KC_HTTP: request is unauthorized!');
                        if (error.url.indexOf('/authorize') === -1) {
                            // auth error handling, observing for authorization
                            return new rxjs_Rx.Observable(function (observer) {
                                if (error.headers.get('WWW-Authenticate') !== null) {
                                    // requesting authorization to KC server
                                    KeycloakAuthorization.authorize(error.headers.get('WWW-Authenticate')).subscribe(function (token) {
                                        // notifying observers for authz result token
                                        observer.next(token);
                                    });
                                }
                                else {
                                    console.warn('WWW-Authenticate header not found' + error.headers.get('WWW-Authenticate'));
                                }
                            });
                        }
                    }
                    else {
                        rxjs_Rx.Observable.throw('server error');
                    }
                });
            }).flatMap(function (res) {
                // Http Response or Authz token
                if (res instanceof _angular_http.Response) {
                    // Http response
                    return new rxjs_Rx.Observable(function (observer) {
                        return observer.next(res);
                    });
                }
                else {
                    // Authorization token
                    Keycloak.accessToken = res;
                    count = count + 1;
                    // retrying request with new token
                    console.info('KC_HTTP: retrying request with new authorization token');
                    return _this.configureRequest(url, count, options);
                }
            });
        }
    };
    // to add 'Authorization' header
    KeycloakHttp.prototype.setHeaders = function (options) {
        return new rxjs_Rx.Observable(function (observer) {
            if (options.withCredentials) {
                console.info('adding headers with options ' + options);
                var token_1 = Keycloak.accessToken;
                if (Keycloak.refreshToken) {
                    console.info('checking token');
                    Keycloak.updateToken(5).subscribe(function (res) {
                        token_1 = res;
                        if (!options.headers) {
                            options.headers = new _angular_http.Headers();
                        }
                        console.info('returning an updated token');
                        options.headers.set('Authorization', 'Bearer ' + token_1);
                        observer.next(options);
                    });
                }
                else {
                    if (!options.headers) {
                        options.headers = new _angular_http.Headers();
                    }
                    console.info('returning the existing token ');
                    options.headers.set('Authorization', 'Bearer ' + token_1);
                    observer.next(options);
                }
            }
            else {
                observer.next(options);
            }
        });
    };
    // Observable on service status.
    // If true, keycloakHttp is ready to handle requests
    KeycloakHttp.readyBehaviourSubject = new rxjs_Rx.BehaviorSubject(false);
    KeycloakHttp.readyObs = KeycloakHttp_1.readyBehaviourSubject.asObservable();
    KeycloakHttp = KeycloakHttp_1 = __decorate$1([
        _angular_core.Injectable()
    ], KeycloakHttp);
    return KeycloakHttp;
    var KeycloakHttp_1;
}(_angular_http.Http));

var __decorate$3 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
function keycloakHttpFactory(backend, defaultOptions, keycloakAuth, keycloak) {
    return new KeycloakHttp(backend, defaultOptions, keycloak, keycloakAuth);
}

var Ng2KeycloakModule = (function () {
    function Ng2KeycloakModule() {
    }
    Ng2KeycloakModule = __decorate$3([
        _angular_core.NgModule({
            imports: [_angular_http.HttpModule],
            declarations: [],
            providers: [Keycloak, KeycloakAuthorization, KeycloakHttp,
                { provide: _angular_http.Http,
                    useFactory: keycloakHttpFactory,
                    deps: [_angular_http.XHRBackend, _angular_http.RequestOptions, Keycloak, KeycloakAuthorization]
                }],
            exports: []
        })
    ], Ng2KeycloakModule);
    return Ng2KeycloakModule;
}());

exports.Keycloak = Keycloak;
exports.KeycloakHttp = KeycloakHttp;
exports.KeycloakAuthorization = KeycloakAuthorization;
exports.Ng2KeycloakModule = Ng2KeycloakModule;

Object.defineProperty(exports, '__esModule', { value: true });

})));
