/*
 * Copyright 2017 ebondu and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Injectable } from '@angular/core';
import { Http, RequestMethod, Response, Headers } from '@angular/http';
import { Observable, BehaviorSubject } from 'rxjs/Rx';
import 'rxjs/operator/map';
import 'rxjs/operator/filter';
import 'rxjs/operator/catch';
import { KeycloakAuthorization } from '../services/keycloak.auth.service';
import { Keycloak } from '../services/keycloak.core.service';
/**
 * An Angular http proxy supporting Keycloak auth & authz.
 * Authenticate user, manage tokens and add authorization header to access to remote Keycloak protected resources.
 */
var KeycloakHttp = (function (_super) {
    __extends(KeycloakHttp, _super);
    function KeycloakHttp(backend, defaultOptions, keycloak, keycloakAuth) {
        var _this = _super.call(this, backend, defaultOptions) || this;
        _this.keycloak = keycloak;
        _this.keycloakAuth = keycloakAuth;
        _this.MAX_UNAUTHORIZED_ATTEMPTS = 2;
        Keycloak.http = new Http(backend, defaultOptions);
        _this.keycloak.init({});
        _this.keycloakAuth.init();
        return _this;
    }
    KeycloakHttp_1 = KeycloakHttp;
    KeycloakHttp.prototype.get = function (url, options) {
        // console.info("GET");
        options = options || { withCredentials: false };
        options.method = RequestMethod.Get;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.post = function (url, body, options) {
        options = options || { withCredentials: false };
        options.method = RequestMethod.Post;
        options.body = body;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.put = function (url, body, options) {
        options = options || { withCredentials: false };
        options.method = RequestMethod.Put;
        options.body = body;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.delete = function (url, options) {
        options = options || { withCredentials: false };
        options.method = RequestMethod.Delete;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.patch = function (url, body, options) {
        options = options || { withCredentials: false };
        options.method = RequestMethod.Patch;
        options.body = body;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.head = function (url, options) {
        options = options || { withCredentials: false };
        options.method = RequestMethod.Head;
        return this.configureRequest(url, 1, options);
    };
    KeycloakHttp.prototype.configureRequest = function (url, count, options) {
        var _this = this;
        if (options.withCredentials && !KeycloakHttp_1.readyBehaviourSubject.getValue()) {
            KeycloakAuthorization.initializedObs.take(1).filter(function (init) { return init === true; }).subscribe(function () {
                console.info('KC_HTTP: keycloak authz initialized...');
            });
            Keycloak.initializedObs.take(1).filter(function (init) { return init === true; }).subscribe(function () {
                if (!Keycloak.authenticatedBehaviourSubject.getValue()) {
                    console.info('KC_HTTP: keycloak initialized, go login...');
                    Keycloak.login(true);
                }
            });
            Keycloak.authenticatedObs.take(2).filter(function (auth) { return auth === true; }).subscribe(function () {
                console.info('KC_HTTP: authentication done...');
                KeycloakHttp_1.readyBehaviourSubject.next(true);
            });
            return KeycloakHttp_1.readyObs.take(2).filter(function (ready) { return ready === true; }).flatMap(function (ready) {
                console.info('KC_HTTP: keycloak is now http ready, re-attempting request...');
                return _this.configureRequest(url, count, options);
            });
        }
        else {
            // KC is ready, getting authorization header
            return this.setHeaders(options).flatMap(function (options) {
                console.info('KC_HTTP: using headers ' + options);
                // calling http with headers
                return _super.prototype.request.call(_this, url, options).catch(function (error) {
                    // error handling
                    var status = error.status;
                    if ((status === 403 || status === 401) && count < _this.MAX_UNAUTHORIZED_ATTEMPTS) {
                        console.info('KC_HTTP: request is unauthorized!');
                        if (error.url.indexOf('/authorize') === -1) {
                            // auth error handling, observing for authorization
                            return new Observable(function (observer) {
                                if (error.headers.get('WWW-Authenticate') !== null) {
                                    // requesting authorization to KC server
                                    KeycloakAuthorization.authorize(error.headers.get('WWW-Authenticate')).subscribe(function (token) {
                                        // notifying observers for authz result token
                                        observer.next(token);
                                    });
                                }
                                else {
                                    console.warn('WWW-Authenticate header not found' + error.headers.get('WWW-Authenticate'));
                                }
                            });
                        }
                    }
                    else {
                        Observable.throw('server error');
                    }
                });
            }).flatMap(function (res) {
                // Http Response or Authz token
                if (res instanceof Response) {
                    // Http response
                    return new Observable(function (observer) {
                        return observer.next(res);
                    });
                }
                else {
                    // Authorization token
                    Keycloak.accessToken = res;
                    count = count + 1;
                    // retrying request with new token
                    console.info('KC_HTTP: retrying request with new authorization token');
                    return _this.configureRequest(url, count, options);
                }
            });
        }
    };
    // to add 'Authorization' header
    KeycloakHttp.prototype.setHeaders = function (options) {
        return new Observable(function (observer) {
            if (options.withCredentials) {
                console.info('adding headers with options ' + options);
                var token_1 = Keycloak.accessToken;
                if (Keycloak.refreshToken) {
                    console.info('checking token');
                    Keycloak.updateToken(5).subscribe(function (res) {
                        token_1 = res;
                        if (!options.headers) {
                            options.headers = new Headers();
                        }
                        console.info('returning an updated token');
                        options.headers.set('Authorization', 'Bearer ' + token_1);
                        observer.next(options);
                    });
                }
                else {
                    if (!options.headers) {
                        options.headers = new Headers();
                    }
                    console.info('returning the existing token ');
                    options.headers.set('Authorization', 'Bearer ' + token_1);
                    observer.next(options);
                }
            }
            else {
                observer.next(options);
            }
        });
    };
    // Observable on service status.
    // If true, keycloakHttp is ready to handle requests
    KeycloakHttp.readyBehaviourSubject = new BehaviorSubject(false);
    KeycloakHttp.readyObs = KeycloakHttp_1.readyBehaviourSubject.asObservable();
    KeycloakHttp = KeycloakHttp_1 = __decorate([
        Injectable()
    ], KeycloakHttp);
    return KeycloakHttp;
    var KeycloakHttp_1;
}(Http));
export { KeycloakHttp };
//# sourceMappingURL=keycloak.http.service.js.map